#!/bin/sh

#это скрипт автоматического обновления
echo $(date +%Y_%m_%d:%H:%M:%S)" START Update script" >> /var/log/updateLog


########################################
#  Check for revert on prev version    #
########################################
# if downloaded update, can't revert version
if [ -d /download/ ]; then
	if [ -f "/flags/flagRevert" ]; then
		echo $(date +%Y_%m_%d:%H:%M:%S)" Cancel revert, because update downloaded!" >> /var/log/updateLog
		rm /flags/flagRevert
		sync
		sync
	fi
fi
# check that revert avaliable
# (exist revert directory + flagRevert=1 + version revert in config)
/userScripts/revertVersionScripts/isRevertAvaliable.sh
if [ $? == 0 ]; then
	revertVersion=$(/userScripts/revertVersionScripts/sqlGetRevertVer.sh)
	echo $(date +%Y_%m_%d:%H:%M:%S)" Start revert on version $revertVersion!" 
	/userScripts/revertVersionScripts/revertVersionStart.sh
	sync
	sync
	reboot
	return 0
fi
########################################

#does update exist?
if ! [ -d /download/ ]; then
  return 0
fi

#if update avaliable now(shift not opened in fiscal stage)
isAvaliable=$(./isUpdAvaliable)
if  [ $isAvaliable == 0 ]; then
  echo $(date +%Y_%m_%d:%H:%M:%S)" Update is not avaliable now, shift is opened!"  >> /var/log/updateLog
  return 0
fi


cd /

rm -r /flags/
rm -r /updateBackup

mkdir updateBackup
touch /updateBackup/MANIFEST

echo $(date +%Y_%m_%d:%H:%M:%S)" Unpack update start!" >> /var/log/updateLog

cd /download

#check checksum
md5sum fisGoUpdate.tar > ourCsFile;
read ourCs trash < ourCsFile;
read inetCs < cs;

if [ "$inetCs" != "$ourCs" ]; then
	echo $(date +%Y_%m_%d:%H:%M:%S)" CS error, update aborted!" >> /var/log/updateLog
		
	#remove download directory
	rm -r /download/
	
	exit 0
fi

cd /download
mv fisGoUpdate.tar fisGoUpdate.tar.gz
gunzip fisGoUpdate.tar.gz
tar xvf fisGoUpdate.tar -C /download/

#tar unpack result
if [ $? != 0 ]; then
	echo $(date +%Y_%m_%d:%H:%M:%S)" Unpack update failed!" >> /var/log/updateLog
	return 1
else
	echo $(date +%Y_%m_%d:%H:%M:%S)" Unpack update success!" >> /var/log/updateLog
fi

#remove archive
rm fisGoUpdate.tar

#kill all process from manifest
#read all strings from file
updateFailure=false;
cat MANIFEST | while read tag val1 val2;
do

	if [ $tag == "process" ]; then
		echo "process $val1" >> /updateBackup/MANIFEST
		
		killall -9 $val1;
		
		if [ $? != 0 ]; then
			echo $(date +%Y_%m_%d:%H:%M:%S)" Failed to kill process: $val1" >> /var/log/updateLog
		fi
	fi
	
	if [ $tag == "file" ]; then
		#if file is new
		if ! [ -d $val2 ]; then 
			echo "remove $val1 $val2" >> /updateBackup/MANIFEST
			mkdir -p $val2;
			cp $val1 $val2;
			if [ $? != 0 ]; then
				echo $(date +%Y_%m_%d:%H:%M:%S)" Failed to copy $val1 to $val2" >> /var/log/updateLog
				updateFailure=true;
			fi
			sync
			continue
		fi
		
		#if file need to backup
		pathToFile=$val2$val1
		echo $pathToFile
		if ! [ -f $pathToFile ]; then
			#file is new
			echo "remove $val1 $val2" >> /updateBackup/MANIFEST
			
			#test bep available
			if [ "$val1" == "libpilarusFiscal.so" ]; then
				bepAvailable=$(mytest 4 | grep ok)
				if ! [ -z "$bepAvailable" ]; then
					continue
				fi
			fi
			
			cp $val1 $val2;
			if [ $? != 0 ]; then
				echo $(date +%Y_%m_%d:%H:%M:%S)" Failed to copy $val1 to $val2" >> /var/log/updateLog
				updateFailure=true;
			fi			
			sync
			continue
		else
			#old file exits
			echo "file $val1 $val2" >> /updateBackup/MANIFEST
			cp $pathToFile /updateBackup/
			sync
			
			#test bep available
			if [ "$val1" == "libpilarusFiscal.so" ]; then
				bepAvailable=$(mytest 4 | grep ok)
				if ! [ -z "$bepAvailable" ]; then
					continue
				fi
			fi
			
			#change files
			cp $val1 $val2;
			if [ $? != 0 ]; then
				echo $(date +%Y_%m_%d:%H:%M:%S)" Failed to copy $val1 to $val2" >> /var/log/updateLog
				updateFailure=true;
			fi
			sync
		fi
	fi
	
	if [ $tag == "remove" ]; then
		echo "file $val1 $val2" >> /updateBackup/MANIFEST
		pathToFile=$val2$val1 
		
		cp $pathToFile /updateBackup/
		
		#remove file
		rm $pathToFile
		
		if [ $? != 0 ]; then
			echo $(date +%Y_%m_%d:%H:%M:%S)" Failed to remove $pathToFile" >> /var/log/updateLog
			updateFailure=true;
		fi
		
		#if directory stay empty
		pathToFile=$val2
		cd $pathToFile
		filesCntInDir=$(ls -1A|wc -l);
		echo $(date +%Y_%m_%d:%H:%M:%S)" files cnt in directory = $filesCntInDir"
		cd -
		if [ $filesCntInDir == 0 ]; then
			rm -rf $pathToFile
		fi
	fi 
done

#remove download directory
rm -r /download/

#correctly update folder removed?
if ! [ -d /download/ ]; then
  echo $(date +%Y_%m_%d:%H:%M:%S)" Update folder removed correctly!" >> /var/log/updateLog
else
  echo $(date +%Y_%m_%d:%H:%M:%S)" Update folder removed incorrectly" >> /var/log/updateLog
fi

#flag Up!
if ! [ -d /flags/ ]; then
    mkdir /flags/
    echo $(date +%Y_%m_%d:%H:%M:%S)" Create /flags " >> /var/log/updateLog
fi   

touch /flags/flagUpd
echo "1" > /flags/flagUpd
sync

############################
#  create new revert ver   #
############################
/userScripts/revertVersionScripts/revertVersionSave.sh
############################

#reboot device!
echo $(date +%Y_%m_%d:%H:%M:%S)" Reboot device after update!" >> /var/log/updateLog 
reboot